'use strict';

import {forEach, objKeys} from "./polyfill";
import {getConfig} from "./config";
import {aeLog} from "./log";

export const transformPropertyMap = {
  translateX: {
    default: '0',
    order: 1,
    name: 'translateX',
    prefix: '(',
    suffix: ')',
    unit: 'px'
  },
  translateY: {
    default: '0',
    order: 2,
    name: 'translateY',
    prefix: '(',
    suffix: ')',
    unit: 'px'
  },
  scaleX: {
    default: '1',
    order: 3,
    name: 'scaleX',
    prefix: '(',
    suffix: ')',
    unit: ''
  },
  scaleY: {
    default: '1',
    order: 4,
    name: 'scaleY',
    prefix: '(',
    suffix: ')',
    unit: ''
  },
  rotate: {
    default: '0 0',
    order: 5,
    name: 'rotate',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  },
  rotateX: {
    default: '0',
    order: 6,
    name: 'rotateX',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  },
  rotateY: {
    default: '0',
    order: 7,
    name: 'rotateY',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  },
};

export const normalPropertyMap = {
  width: {
    default: '0',
    order: -1,
    name: 'width',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  height: {
    default: '0',
    order: -2,
    name: 'height',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  left: {
    default: '0',
    order: -3,
    name: 'left',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  top: {
    default: '0',
    order: -4,
    name: 'top',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  opacity: {
    default: '1',
    order: -5,
    name: 'opacity',
    prefix: ':',
    suffix: ';',
    unit: ''
  },
  layerTransformOrigin: {
    default: '50% 50%',
    order: -6,
    name: 'transform-origin',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  vectorTransformOrigin: {
    default: '50% 50%',
    order: -7,
    name: 'transform-origin',
    prefix: ':',
    suffix: ';',
    unit: '%'
  },
  perspective: {
    default: 'none',
    order: -8,
    name: 'perspective',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  easing: {
    default: null,
    order: -9,
    name: 'animation-timing-function',
    prefix: ':',
    suffix: ';',
    unit: ''
  },
  background: {
    default: null,
    order: -10,
    name: 'background',
    prefix: ':',
    suffix: ';',
    unit: ''
  },
};

export function getRandomColor() {
  let r = Math.floor(Math.random() * 256).toString(16);
  let g = Math.floor(Math.random() * 256).toString(16);
  let b = Math.floor(Math.random() * 256).toString(16);
  const fillColor = (c) => {
    return c.length < 2 ? '0' + c : c;
  };
  return '#' + fillColor(r) + fillColor(g) + fillColor(b) + ';';
}

function scaleNumber(value, unit) {
  if (!isNumber(value)) {
    return value;
  }
  const scaleRatio = getConfig('IFRAME_SCALE_RATIO', 1);
  return numberShaking(value * scaleRatio, 1);
}

function getArrayStringProperty(value, typeObj) {
  let tempValue = '';
  let arr = [value];
  if (typeof value === 'string') {
    arr = value.split(' ');
  }

  if (arr.length > 1) {
    forEach(arr, (child) => {
      tempValue += `${scaleNumber(child, typeObj.unit)}${Number(child) ? typeObj.unit : ''} `;
    });
  } else {
    tempValue = `${scaleNumber(arr[0], typeObj.unit)}${Number(arr[0]) ? typeObj.unit : ''}`;
  }
  return tempValue;
}

function mergeTransformProperty(str) {
  if (!str) return '';
  return str.replace(/translateX\((.*?)\) translateY\((.*?)\)/, (w, w1, w2) => (`translate(${w1}, ${w2})`))
  .replace(/scaleX\((.*?)\) scaleY\((.*?)\)/, (w, w1, w2) => {
    if (Number(w1) === Number(w2)) {
      return `scale(${w1})`;
    } else {
      return `scale(${w1}, ${w2})`;
    }
  })
  .replace(/rotateX\((.*?)\) rotateY\((.*?)\)/, (w, w1, w2) => {
    if (Number(w1) === Number(w2)) {
      return `rotate(${w1})`;
    } else {
      return `rotate(${w1}, ${w2})`;
    }
  });
}

export function getPropertyCss(obj, isLastKeyframe) {
  let transformPropertyCss = '';
  let normalPropertyCss = '';
  // 对key 进行排序... 为了保证rotate是最后之类的...
  let keyList = objKeys(obj);
  keyList.sort((before, after) => {
    const keyOrder = (key) => {
      if (transformPropertyMap[key]) {
        return transformPropertyMap[key].order;
      } else if (normalPropertyMap[key]) {
        return normalPropertyMap[key].order;
      } else {
        return 0;
      }
    };
    return keyOrder(before) - keyOrder(after);
  });
  aeLog('keyList', keyList);
  forEach(keyList, (key, index) => {
    if (transformPropertyMap[key] && (isLastKeyframe || transformPropertyMap[key].default !== obj[key])) {
      // 当前帧不是最后一帧（最后一帧一定要显示效果） || 当前帧的transform属性不是默认值
      if (!transformPropertyCss) {
        transformPropertyCss += 'transform: ';
      }
      transformPropertyCss += `${transformPropertyMap[key].name}${transformPropertyMap[key].prefix}${getArrayStringProperty(obj[key], transformPropertyMap[key])}${transformPropertyMap[key].suffix} `;
    }
    if (normalPropertyMap[key] && obj[key] !== undefined) {
      if (index > 0) {
        normalPropertyCss += '\n';
      }
      if (key === 'transformOrigin') {
        normalPropertyCss += `${normalPropertyMap[key].name}${normalPropertyMap[key].prefix} ${getArrayStringProperty(obj[key], normalPropertyMap[key])}${normalPropertyMap[key].suffix}`;
      } else {
        normalPropertyCss += `${normalPropertyMap[key].name}${normalPropertyMap[key].prefix} ${getArrayStringProperty(obj[key], normalPropertyMap[key])}${normalPropertyMap[key].suffix}`;
      }
    }
  });
  // transformPropertyCss 正则表达式处理
  transformPropertyCss = mergeTransformProperty(transformPropertyCss);
  return {transformPropertyCss, normalPropertyCss};
}

export function isNumber(value) {
  return typeof Number(value) === 'number' && !isNaN(value);
}

export function numberShaking(num, digitCount = 1) {
  if (!isNumber(num)) {
    return num;
  }
  return Number(Number(num).toFixed(digitCount));
}

